@@ -1,23 +1,29 @@
 import os
 import random
 import sys
-from functools import partial
 from time import sleep
 
 import keyring
-import requests
 from baseconv import base62
 from configobj import ConfigObj
+from furl import furl
 from loguru import logger
+from requests_cache import CachedSession
+from pathlib import Path
 
 logger.remove()
 logger.add(sys.stdout, colorize=True)
 
-_configpath = os.environ.get('XDG_CACHE_HOME') or os.environ.get('HOME')
-CONFIG_FILE = os.path.join(_configpath, 'sinaspider.ini')
+xdg_cache_home = os.environ.get('XDG_CACHE_HOME') or os.environ.get('HOME')
+CONFIG_FILE = os.path.join(xdg_cache_home, 'sinaspider.ini')
+weibo_api_url = furl(url='https://m.weibo.cn', path='api/container/getIndex')
 
+headers = {
+    "User_Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:90.0) Gecko/20100101 Firefox/90.0",
+    "Cookie": keyring.get_password('sinaspider', 'cookie')
+}
 
-def config(account_id=None, database_name=None, write_xmp=None):
+def get_config(account_id=None, database_name=None, write_xmp=None, download_dir=None):
     """
     写入并读取配置
     Args:
@@ -27,39 +33,45 @@ def config(account_id=None, database_name=None, write_xmp=None):
     Returns:
         [dict]: 当前的配置信息
     """
-    default = {'database_name': 'sina', 'write_xmp': False}
+    overwrite = {k: v for k, v in locals().items() if v is not None}
+    default = {'database_name': 'sina', 'write_xmp': False,
+               'download_dir': Path.home()/'Downloads/sinaspider'}
     config = ConfigObj(CONFIG_FILE)
-    config.update(default | config)
-    if account_id is not None:
-        config['account_id'] = account_id
-    if database_name is not None:
-        config['database_name'] = database_name
-    if write_xmp is not None:
-        assert isinstance(write_xmp, bool)
-        config['write_xmp'] = write_xmp
+    config.update(default | config | overwrite)
     config.write()
     return config
 
 
-headers = {
-    "User_Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:90.0) Gecko/20100101 Firefox/90.0",
-    "Cookie": keyring.get_password('sinaspider', 'cookie')
-}
+def get_url(url, expire_after=0):
+    session = CachedSession(cache_name=f'{xdg_cache_home}/sinaspider/http_cache',
+                            expire_after=expire_after)
 
-get_url = partial(requests.get, headers=headers)
-
-
-def get_json(**params):
-    url = 'https://m.weibo.cn/api/container/getIndex?'
     while True:
         try:
-            r = get_url(url, params=params)
+            r = session.get(url, headers=headers)
             break
         except TimeoutError:
             logger.error('Timeout sleep 600seconds and retry...')
             sleep(10 * 60)
 
-    return r.json()
+    return r
+
+
+
+def write_xmp(tags, img):
+    if get_config().as_bool('write_xmp'):
+        try:
+            import exiftool
+        except ModuleNotFoundError as e:
+            logger.warning(
+                'exiftool not installed, cannot write xmp info to img')
+            return
+    else:
+        return
+
+    with exiftool.ExifTool() as et:
+        et.set_tags(tags, str(img))
+        Path(img).with_name(Path(img).name+'_original').unlink()
 
 
 class Pause:
@@ -67,11 +79,11 @@ class Pause:
 
         self.page_config = dict(
             awake=0,
-            stop=random.randint(3, 5),
+            stop=random.randint(4, 8),
             visited=0,
             level={
-                'short': 5,
-                'break': 20,
+                'short': 3,
+                'break': 10,
                 'long': 50
             },
             break_freq=100
@@ -113,7 +125,9 @@ class Pause:
 
     @staticmethod
     def _sleep(sleep_time):
-        sleep_time = random.randint(int(0.5 * sleep_time), int(1.5 * sleep_time))
+        sleep_time = random.randint(
+            int(0.5 * sleep_time), int(1.5 * sleep_time))
+        logger.info(f'sleeping {sleep_time} second(s)')
 
         for i in range(sleep_time):
             print(f'sleep {i}/{sleep_time}', end='\r')
